//! This mod wraps the event Ids
use serde::{
    de::{self, Deserializer},
    ser::{self, Serializer},
    Deserialize,
};
use std::ops::Deref;

/// Event Id
///
/// Event Id are raw 32 bytes and 64-character length hex encoded to JSON
#[derive(Debug, Clone, Eq, PartialEq)]
pub struct Id(pub [u8; 32]);

impl Deref for Id {
    type Target = [u8; 32];
    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

impl ToString for Id {
    fn to_string(&self) -> String {
        hex::encode(self.0)
    }
}

impl<'de> Deserialize<'de> for Id {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        let s = <String>::deserialize(deserializer)?;
        let id: [u8; 32] = hex::decode(&s)
            .map_err(|e| de::Error::custom(e.to_string()))?
            .try_into()
            .map_err(|_| de::Error::custom("Invalid length for Id"))?;

        Ok(Self(id))
    }
}

impl ser::Serialize for Id {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(self.to_string().as_str())
    }
}
